diff mbox

[v3] disk: part_efi: add support for the Backup GPT

Message ID 1399320008-3664-1-git-send-email-srae@broadcom.com
State Accepted
Commit ae95fad5af3793ae804b4390727e0f431e5514d7
Headers show

Commit Message

Steve Rae May 5, 2014, 8 p.m. UTC
Check the Backup GPT table if the Primary GPT table is invalid.
Renamed "Secondary GPT" to "Backup GPT" as per:
  UEFI Specification (Version 2.3.1, Errata A)

Signed-off-by: Steve Rae <srae@broadcom.com>
---

Changes in v3:
- renamed: "Secondary GPT" to "Backup GPT"

Changes in v2:
- changed wording: "Alternate" to "Secondary"

 disk/part_efi.c | 22 +++++++++++++++++++---
 doc/README.gpt  | 14 +++++++-------
 2 files changed, 26 insertions(+), 10 deletions(-)

Comments

Tom Rini May 13, 2014, 1:54 a.m. UTC | #1
On Mon, May 05, 2014 at 01:00:08PM -0700, Steve Rae wrote:

> Check the Backup GPT table if the Primary GPT table is invalid.
> Renamed "Secondary GPT" to "Backup GPT" as per:
>   UEFI Specification (Version 2.3.1, Errata A)
> 
> Signed-off-by: Steve Rae <srae@broadcom.com>

Applied to u-boot/master, thanks!
diff --git a/doc/README.gpt b/doc/README.gpt
index f822894..ec0156d 100644
--- a/doc/README.gpt
+++ b/doc/README.gpt
@@ -66,14 +66,14 @@  GPT brief explanation:
 		       |Partition n                      |
 		       |                                 |
 	----------------------------------------------------------
-	LBA -34        |Entry 1|Entry 2| Entry 3| Entry 4| Secondary
-	-------------------------------------------------- (bkp)
-	LBA -33        |Entries 5 - 128                  | GPT
+	LBA -34        |Entry 1|Entry 2| Entry 3| Entry 4| Backup
+	-------------------------------------------------- GPT
+	LBA -33        |Entries 5 - 128                  |
 		       |                                 |
 		       |                                 |
 	LBA -2         |                                 |
 	--------------------------------------------------
-	LBA -1         |Secondary GPT Header             |
+	LBA -1         |Backup GPT Header                |
 	----------------------------------------------------------
 
 For a legacy reasons, GPT's LBA 0 sector has a MBR structure. It is called
@@ -86,7 +86,7 @@  It is possible to define 128 linearly placed partition entries.
 "LBA -1" means the last addressable block (in the mmc subsystem:
 "dev_desc->lba - 1")
 
-Primary/Secondary GPT header:
+Primary/Backup GPT header:
 ----------------------------
 Offset  Size    Description
 
@@ -115,7 +115,7 @@  IMPORTANT:
 
 GPT headers and partition entries are protected by CRC32 (the POSIX CRC32).
 
-Primary GPT header and Secondary GPT header have swapped values of "Current LBA"
+Primary GPT header and Backup GPT header have swapped values of "Current LBA"
 and "Backup LBA" and therefore different CRC32 check-sum.
 
 CRC32 for GPT headers (field "CRC of header") are calculated up till
@@ -125,7 +125,7 @@  CRC32 for partition entries (field "CRC32 of partition array") is calculated for
 the whole array entry ( Number_of_partition_entries *
 sizeof(partition_entry_size (usually 128)))
 
-Observe, how Secondary GPT is placed in the memory. It is NOT a mirror reflect
+Observe, how Backup GPT is placed in the memory. It is NOT a mirror reflect
 of the Primary.
 
 	   Partition Entry Format:
diff mbox

Patch

diff --git a/disk/part_efi.c b/disk/part_efi.c
index 216a292..c74b7b9 100644
--- a/disk/part_efi.c
+++ b/disk/part_efi.c
@@ -93,7 +93,15 @@  void print_part_efi(block_dev_desc_t * dev_desc)
 	if (is_gpt_valid(dev_desc, GPT_PRIMARY_PARTITION_TABLE_LBA,
 			 gpt_head, &gpt_pte) != 1) {
 		printf("%s: *** ERROR: Invalid GPT ***\n", __func__);
-		return;
+		if (is_gpt_valid(dev_desc, (dev_desc->lba - 1),
+				 gpt_head, &gpt_pte) != 1) {
+			printf("%s: *** ERROR: Invalid Backup GPT ***\n",
+			       __func__);
+			return;
+		} else {
+			printf("%s: ***        Using Backup GPT ***\n",
+			       __func__);
+		}
 	}
 
 	debug("%s: gpt-entry at %p\n", __func__, gpt_pte);
@@ -142,7 +150,15 @@  int get_partition_info_efi(block_dev_desc_t * dev_desc, int part,
 	if (is_gpt_valid(dev_desc, GPT_PRIMARY_PARTITION_TABLE_LBA,
 			gpt_head, &gpt_pte) != 1) {
 		printf("%s: *** ERROR: Invalid GPT ***\n", __func__);
-		return -1;
+		if (is_gpt_valid(dev_desc, (dev_desc->lba - 1),
+				 gpt_head, &gpt_pte) != 1) {
+			printf("%s: *** ERROR: Invalid Backup GPT ***\n",
+			       __func__);
+			return -1;
+		} else {
+			printf("%s: ***        Using Backup GPT ***\n",
+			       __func__);
+		}
 	}
 
 	if (part > le32_to_cpu(gpt_head->num_partition_entries) ||
@@ -252,7 +268,7 @@  int write_gpt_table(block_dev_desc_t *dev_desc,
 	    != pte_blk_cnt)
 		goto err;
 
 	/* recalculate the values for the Second GPT Header */
 	val = le64_to_cpu(gpt_h->my_lba);
 	gpt_h->my_lba = gpt_h->alternate_lba;
 	gpt_h->alternate_lba = cpu_to_le64(val);
@@ -252,7 +268,7 @@  int write_gpt_table(block_dev_desc_t *dev_desc,
 		goto err;
 
-	/* recalculate the values for the Second GPT Header */
+	/* recalculate the values for the Backup GPT Header */
 	val = le64_to_cpu(gpt_h->my_lba);
 	gpt_h->my_lba = gpt_h->alternate_lba;